/*= OX803 Metrix ========================================================= */
/* LabWindows Instrument Driver for OX803.                                 */
/* Original Release: April/97                                              */
/* By:  O. LEMAIRE                                                         */
/* Originally written in C                                                 */
/* Modification History:                                                   */
/*=========================================================================*/
#include "formatio.h"
#include "rs232.h"
#include "ox803.h"


/*= STATIC VARIABLES ======================================================*/
/* port contains the number of the port opened for the instrument module.  */
/* vert_mode contains the current vertical mode of the instrument.         */
/* ox803_err: the error variable for the instrument module                 */
/*=========================================================================*/
int port;
int vert_mode;
int ox803_err;


/*= UTILITY ROUTINES ======================================================*/
int ox803_invalid_short_range (short val, short min, short max, int err_code);
int ox803_invalid_integer_range (int val, int min, int max, int err_code);
int ox803_invalid_longint_range (long val, long min, long max, int err_code);
int ox803_invalid_real_range (double val, double min, double max, int err_code);
int ox803_device_closed (void);
int ox803_send_command (int, int);

/*=========================================================================*/
/* This function opens a com port for the instrument module, queries for   */
/* ID, and initializes the instrument to a known state.                    */
/*=========================================================================*/
int ox803_init (int rs_port)
{
    char identifier[3];

    ox803_err = 0;
    if (ox803_invalid_integer_range (rs_port,1,8,-1) != 0){
        return ox803_err;
    }

    /*   Open And Configure RS232  */
    OpenComConfig (rs_port, "", 9600, 0, 8, 1, 512, 512);
    if (rs232err != 0) {
        ox803_err = rs232err;
        return ox803_err;
    }
    /*   Time-out : 1 Sec   */
    SetComTime (rs_port, 1.0);
    if (rs232err != 0) {
        ox803_err = rs232err+300;
        return ox803_err;
    }
    /*   Sets XON/XOFF ON   */
	SetXMode (rs_port, 1);
    if (rs232err != 0) {
        ox803_err = rs232err+300;
        return ox803_err;
    }
    port = rs_port;

    /*   Identify instrument   */
    if (ox803_send_command (OX803_ASK_ECHO, 0) !=0 ){
        ox803_close();
        return ox803_err;
    }
    ComRdTerm (port, identifier, 03, 04);
    if (identifier[1]!=0x32){
         ox803_close();
         return 223;
    }

    /*   Sets the Instrument with default values  */
    /*    Beam Finder   :   OFF             */
    /*    Vertical mode :   ALT             */
    /*    CH1           : AC    0.1V/Div    */
    /*    Inversion CH2 :   OFF             */
    /*    CH2           : AC    0.1V/Div    */
    /*    Time base     : x10 OFF,TBA=50S/Div,DELAY OFF */
    /*    Trigger       : PTP ON,SLOPE +,AUTO ON,SOURCE CH1,COUPLING DC  */
    vert_mode = 2; /* ALT */
    if (ox803_display (2) != 0){
        return ox803_err;
    }
    if (ox803_channel (0, 0, 7, 0) != 0){
        return ox803_err;
    }
    if (ox803_channel (1, 0, 7, 0) !=0){
        return ox803_err;
    }
    if (ox803_timebase (0, 11, 0) != 0){
        return ox803_err;
    }
    if (ox803_trigger (1, 0, 1, 0, 0) != 0){
        return ox803_err;
    }
    return ox803_err;
}
/*---------------------------------------------------------------------*/
/*                 Instrument Setup subroutine                         */
/*---------------------------------------------------------------------*/
int ox803_setup (int rw_config, char *file)
{
    int     file_id;
    short   i,tab_fonct[19];
    char    buff[20];

    tab_fonct[0] = OX803_RANGE_CH1;
    tab_fonct[1] = OX803_RANGE_CH2;				
    tab_fonct[2] = OX803_COUPLING_CH1;
    tab_fonct[3] = OX803_COUPLING_CH2;
    tab_fonct[4] = OX803_VERTICAL_MODE;
    tab_fonct[5] = OX803_INVERT_CH2;					
    tab_fonct[6] = OX803_RANGE_TIME_BASE;
    tab_fonct[7] = OX803_EXP10;
    tab_fonct[8] = OX803_TRIGGER_SOURCE;
    tab_fonct[9] = OX803_TRIGGER_AUTO;			
    tab_fonct[10] = OX803_TRIGGER_COUPLING;								
    tab_fonct[11] = OX803_TRIGGER_PTP;
    tab_fonct[12] = OX803_TRIGGER_SLOPE;				
    tab_fonct[13] = OX803_TB_DELAY;

    ox803_err = 0;
    if (ox803_device_closed() != 0 ){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (rw_config,0,1,-1) != 0){
        return ox803_err;
    }
    if (!rw_config)  {
        /* Save */
        file_id = OpenFile (file, 2, 0, 1);
        if (file_id ==  -1)  {
            ox803_err =  -2;
            return ox803_err;
        }
        /* Input Buffer Empty */
        FlushInQ (port);
        if (rs232err != 0) {
            ox803_err = rs232err+300;
            return ox803_err;
        }
        if (ox803_send_command (OX803_READ_SETUP, 0) != 0 ){
            return ox803_err;
        }
        i = 0;
        while (i < 14)  {
            ComRdByte (port);
            if (rs232err != 0)  {
                ox803_err = rs232err+300;
                return ox803_err;
            }
            buff[i] = ComRdByte (port);
            if (rs232err != 0)  {
                ox803_err = rs232err+300;
                return ox803_err;
            }
            ComRdByte (port);
            if (rs232err != 0)  {
                ox803_err = rs232err+300;
                return ox803_err;
            }
            i++;
        }

        if (WriteFile (file_id, buff, 14) == -1){
            ox803_err =  230;
            return ox803_err;
        }
        if (CloseFile (file_id) == -1){
            ox803_err = -2 ;
            return ox803_err;
        }
    }
    else  {
        /* Recall */
        file_id = OpenFile (file, 1, 0, 1);
        if (file_id ==  -1)  {
            ox803_err =  -2;
            return ox803_err;
        }
        if (ReadFile (file_id, buff, 19) ==  -1)  {
            ox803_err =  231;
            return ox803_err;
        }
        i = 0;
        vert_mode = buff[1] - 0x30;
        while (i < 14)  {
            if (ox803_send_command (tab_fonct[i], buff[i] - 0x30) != 0 ){
                return ox803_err;
            }
            i++;
        }
        if (CloseFile (file_id) == -1 ){
            ox803_err =  -2;
            return ox803_err;
        }

    }
    return ox803_err;
}
/*---------------------------------------------------------------------*/
/*                  Autoset subroutine                                 */
/*---------------------------------------------------------------------*/
int ox803_autoset (int search)
{
    ox803_err = 0;
    if (ox803_device_closed() != 0 ){
        return ox803_err ;
    }
    if (ox803_invalid_integer_range (search,0,1,-1) != 0){
        return ox803_err;
    }
	if(search==0){
		if (ox803_send_command(OX803_COM_AUTOSET,2) !=0 ){
        	return ox803_err;
    	}
    }
    else {
		if (ox803_send_command(OX803_COM_AUTOSET,1) !=0 ){
        	return ox803_err;
    	}
    }
    
    return ox803_err;
}
/*---------------------------------------------------------------------*/
/*                  Display settings subroutine                        */
/*---------------------------------------------------------------------*/
int ox803_display (int mod)
{
    ox803_err = 0;
    if (ox803_device_closed() != 0 ){
        return ox803_err ;
    }
    if (ox803_invalid_integer_range (mod,0,5,-1) != 0){
        return ox803_err;
    }
    if (ox803_send_command (OX803_VERTICAL_MODE, mod) != 0){
        return ox803_err;
    }
    vert_mode = mod;
    return ox803_err;
}
/*---------------------------------------------------------------------*/
/*                 Channel Settings subroutine                         */
/*---------------------------------------------------------------------*/
int ox803_channel (int channel_num, int coupling, int sensibility, int inversion)
{
    ox803_err = 0;
    if (ox803_device_closed() != 0 ){
        return ox803_err ;
    }
    if (ox803_invalid_integer_range (channel_num,0,1,-1) != 0){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (coupling,0,2,-2) != 0){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (sensibility,0,14,-3) != 0){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (inversion,0,1,-4) != 0){
        return ox803_err;
    }
    if (!channel_num){
        if (ox803_send_command (OX803_COUPLING_CH1, coupling) != 0)
            return ox803_err;
    }
    else{
        if (ox803_send_command (OX803_COUPLING_CH2, coupling) != 0)
            return ox803_err;
    }
    if (!channel_num){
        if (ox803_send_command (OX803_RANGE_CH1, sensibility) != 0)
            return ox803_err;
    }
    else{
        if (ox803_send_command (OX803_RANGE_CH2, sensibility) != 0){
            return ox803_err;
        }
    }
    /* Channel invert */
    if (ox803_send_command (OX803_INVERT_CH2, inversion) !=0){
        return ox803_err;
    }

    return ox803_err;
}
/*---------------------------------------------------------------------*/
/*                     Time base subroutine                            */
/*---------------------------------------------------------------------*/
int ox803_timebase (int expend,int range,int dlay)
{

    ox803_err = 0;
    if (ox803_device_closed() != 0 ){
        return ox803_err ;
    }
    if (ox803_invalid_integer_range (expend,0,1,-1) != 0){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (range,0,21,-2) != 0){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (dlay,0,2,-3) != 0){
        return ox803_err;
    }
    if (ox803_send_command (OX803_EXP10, expend) != 0){
        return ox803_err;
    }
    if (ox803_send_command (OX803_RANGE_TIME_BASE, range) != 0){
        return ox803_err;
    }
    if (ox803_send_command (OX803_TB_DELAY, dlay) != 0){
        return ox803_err;
    }
    return ox803_err;
}
/*---------------------------------------------------------------------*/
/*                     Trigger subroutine                              */
/*---------------------------------------------------------------------*/
int ox803_trigger (int ptp,int slope,int mod_auto,int source,int coupling)
{
    ox803_err = 0;
    if (ox803_device_closed() != 0 ){
        return ox803_err ;
    }
    if (ox803_invalid_integer_range (ptp,0,1,-1) != 0){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (slope,0,1,-2) != 0){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (mod_auto,0,1,-3) != 0){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (source,0,4,-4) != 0){
        return ox803_err;
    }
    if (ox803_invalid_integer_range (coupling,0,5,-5) != 0){
        return ox803_err;
    }
    /* if TEST mode, error= -6 */
    if (vert_mode == 5)  {
        ox803_err =  -6;
        return ox803_err ;
    }
    if (ox803_send_command (OX803_TRIGGER_SOURCE, source) != 0){
        return ox803_err;
    }
    if (source != 3){
        if (ox803_send_command (OX803_TRIGGER_COUPLING, coupling) != 0){
            return ox803_err;
        }
    }
    /* if SOURCE LINE, COUPLING OFF */
    else {
        if (ox803_send_command (OX803_TRIGGER_COUPLING, 6) != 0){
            return ox803_err;
        }
    }
    if (ox803_send_command (OX803_TRIGGER_PTP, ptp) != 0){
        return ox803_err;
    }
    if (ox803_send_command (OX803_TRIGGER_AUTO, mod_auto) != 0){
        return ox803_err;
    }
    if (ox803_send_command (OX803_TRIGGER_SLOPE, slope) != 0){
        return ox803_err;
    }
    return ox803_err;
}

/*----------------------------------------------------------------------*/
/* This function closes the port for the instrument module and sets the */
/* port to zero.                                                        */
/*----------------------------------------------------------------------*/
int ox803_close (void)
{
    /*  Check for device closed */
    if (ox803_device_closed() !=0){
        return ox803_err;
    }

    /* Wait for Empty output buffer */
    while ( GetOutQLen(port) != 0) {};
    /* Close the com port.  If error, set ox803_err = rs232err+300.*/
    CloseCom(port);
    if (rs232err != 0) {
        ox803_err = rs232err+300;
        return ox803_err;
    }
    port = 0;
    return ox803_err;
}
/* = UTILITY ROUTINES =====================================================*/
/*=========================================================================*/
/* Function: Invalid Short Range                                           */
/* Purpose:  This function checks a short to see if it lies between a      */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox803_invalid_short_range (short val, short min, short max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox803_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Integer Range                                         */
/* Purpose:  This function checks an integer to see if it lies between a   */
/*           minimum and maximum value.  If the value is out of range, set */
/*           the global error variable to the value err_code.  If the      */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox803_invalid_integer_range (int val, int min, int max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox803_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Invalid Long Integer Range                                    */
/* Purpose:  This function checks a long integer to see if it lies between */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.  The return value is equal to the     */
/*           global error value.                                           */
/*=========================================================================*/
int ox803_invalid_longint_range (long val, long min, long max, int err_code)
{
    if (val < min || val > max) {
        ox803_err = err_code;
        return -1;
    }
    return 0;
}
/*=========================================================================*/
/* Function: Invalid Real Range                                            */
/* Purpose:  This function checks a real number to see if it lies between  */
/*           a minimum and maximum value.  If the value is out of range,   */
/*           set the global error variable to the value err_code.  If the  */
/*           value is OK, error = 0.                                       */
/*=========================================================================*/
int ox803_invalid_real_range (double val, double min, double max, int err_code)
{
  if ((val < min) || (val > max)) {
    ox803_err = err_code;
    return -1;
  }
  return 0;
}
/*=========================================================================*/
/* Function: Device Closed                                                 */
/* Purpose:  This function checks to see if the module has been            */
/*           initialized.  If the device has not been opened, a 1 is       */
/*           returned, 0 otherwise.                                        */
/*=========================================================================*/
int ox803_device_closed (void)
{
    if (port== 0){
        ox803_err = 232;
        return -1;
    }
    return 0 ;

}
/*=========================================================================*/
/* Function: Send Command                                                  */
/* Purpose: This function sends a command to the instrument. If the        */
/*          scope is not initialized, an error 232 is returned.            */
/*=========================================================================*/
int ox803_send_command (int function, int parameter)
{
    ox803_err = 0;
    if (port != 0)  {
        parameter += 0x30;
        if (ComWrtByte (port, function) ==0){
            ox803_err = 230;
            return ox803_err ;
        }
        if (ComWrtByte (port, parameter) == 0){
            ox803_err = 230;
            return ox803_err ;
        }
        if (ComWrtByte (port, 04) ==0){
            ox803_err = 230;
            return ox803_err ;
        }
    }
    else{
        ox803_err = 232;
    }
    return ox803_err ;
}
/*= THE END ===============================================================*/
